@@ -1,12 +1,15 @@
-#include <stdint.h>
+#ifndef _JSON_H_
+#define _JSON_H_
+
+#include <stddef.h>
 
 enum JSONType {
-	TYPE_ARRAY     = 'a',
-	TYPE_OBJECT    = 'o',
-	TYPE_STRING    = 's',
-	TYPE_BOOL      = 'b',
-	TYPE_NULL      = '?',
-	TYPE_NUMBER    = 'n'
+	JSON_TYPE_ARRAY  = 'a',
+	JSON_TYPE_OBJECT = 'o',
+	JSON_TYPE_STRING = 's',
+	JSON_TYPE_BOOL   = 'b',
+	JSON_TYPE_NULL   = '?',
+	JSON_TYPE_NUMBER = 'n'
 };
 
 enum JSONError {
@@ -24,3 +27,4 @@ struct json_node {
 };
 
 int parsejson(void (*cb)(struct json_node *, size_t, const char *));
+#endif
