@@ -14,6 +14,9 @@
 
 #include "json.h"
 
+/* control-character in the ASCII range 0-127: compatible with UTF-8 */
+#define ISCNTRL(c) ((c) < ' ' || (c) == 0x7f)
+
 static int nflag = 0; /* -n flag: show indices count for arrays */
 static int rflag = 0; /* -r flag: show all control-characters */
 static int fs = '\t', rs = '\n';
@@ -31,7 +34,7 @@ tsv_printvalue(const char *s)
 		case '\t': putchar('\\'); putchar('t'); break;
 		default:
 			/* ignore other control chars */
-			if (!rflag && iscntrl((unsigned char)*s))
+			if (!rflag && ISCNTRL((unsigned char)*s))
 				continue;
 			putchar(*s);
 		}
@@ -52,7 +55,7 @@ rs_printvalue(const char *s)
 			break;
 		default:
 			/* ignore other control chars */
-			if (!rflag && iscntrl((unsigned char)*s))
+			if (!rflag && ISCNTRL((unsigned char)*s))
 				continue;
 			putchar(*s);
 		}
@@ -110,6 +113,11 @@ processnode(struct json_node *nodes, size_t depth, const char *value)
 	putchar(fs);
 	printvalue(value);
 	putchar(rs);
+
+	if (ferror(stdout)) {
+		fprintf(stderr, "write error: <stdout>\n");
+		exit(2);
+	}
 }
 
 int
@@ -138,12 +146,20 @@ readchar(const char *s)
 		return *s;
 	} else if (*s == '\\') {
 		s++;
+		if (*s == 'x')
+			return readnum(++s, 16); /* hexadecimal */
+		else if (isdigit((unsigned char)*s))
+			return readnum(s, 8); /* octal */
+
+		if (*(s + 1)) {
+			fprintf(stderr, "unsupported format\n");
+			exit(3);
+		}
 		switch (*s) {
 		case '\\': return '\\';
 		case 't': return '\t';
 		case 'n': return '\n';
 		case 'r': return '\r';
-		case 'x': return readnum(++s, 16); /* hexadecimal */
 		default:
 			fprintf(stderr, "unsupported escape character\n");
 			exit(3);
@@ -209,5 +225,14 @@ nextarg:;
 		return 1;
 	}
 
+	if (ferror(stdin)) {
+		fprintf(stderr, "read error: <stdin>\n");
+		return 2;
+	}
+	if (fflush(stdout) || ferror(stdout)) {
+		fprintf(stderr, "write error: <stdout>\n");
+		return 2;
+	}
+
 	return 0;
 }
